package uts.sdk.modules.lemonFilePicker;
import kotlinx.coroutines.async;
import kotlinx.coroutines.CoroutineScope;
import kotlinx.coroutines.Deferred;
import kotlinx.coroutines.Dispatchers;
import io.dcloud.uts.Map;
import io.dcloud.uts.*;
import io.dcloud.uts.UTSAndroid;
import androidx.core.app.ActivityCompat;
import android.content.Context;
import android.content.Intent;
import android.content.ContentUris;
import android.content.ContentResolver;
import android.provider.Settings;
import android.net.Uri;
import android.os.FileUtils;
import android.provider.DocumentsContract;
import android.provider.MediaStore;
import java.io.File;
import java.io.FileOutputStream;
import android.app.Activity;
import android.Manifest;
import android.database.Cursor;
import android.os.Build;
import android.os.Environment;
open class InfoOptions (
    open var permission: Boolean? = null,
    open var mimetype: String? = null,
    open var success: ((res: Any) -> Unit)? = null,
    open var fail: ((res: Any) -> Unit)? = null,
    open var complete: ((res: Any) -> Unit)? = null,
)
fun fileSelect(options: InfoOptions) {
    UTSAndroid.onAppActivityRequestPermissionsResult(fun(requestCode: Number, permissions: MutableList<String>, grantResults: MutableList<Number>){
        UTSAndroid.offAppActivityRequestPermissionsResult();
        if (requestCode !== 1001 && grantResults[0] === -1) {
            options.fail?.invoke(object : UTSJSONObject() {
                var code = "1001"
                var errMsg = "fileselect:fail"
                var detail = "未授权文件读取权限"
            });
            val permission = options.permission;
            if (permission != null && permission) {
                val intent = Intent();
                intent.setAction(Settings.ACTION_APPLICATION_DETAILS_SETTINGS);
                val uri = Uri.fromParts("package", UTSAndroid.getUniActivity()?.getPackageName(), "");
                intent.setData(uri);
                UTSAndroid.getUniActivity()?.startActivity(intent);
            }
        } else fileSelectStart(options);
    }
    );
    ActivityCompat.requestPermissions(UTSAndroid.getUniActivity()!!, arrayOf(Manifest.permission.READ_EXTERNAL_STORAGE), 1001);
}
fun fileSelectStart(options: InfoOptions) {
    val context = UTSAndroid.getAppContext();
    if (context != null) {
        val intent = Intent(Intent.ACTION_GET_CONTENT);
        intent.setType("*/*");
        intent.addCategory(Intent.CATEGORY_OPENABLE);
        if (options.mimetype != null) intent.setType(options.mimetype);
        UTSAndroid.getUniActivity()?.startActivityForResult(intent, 110);
        UTSAndroid.onAppActivityResult(fun(requestCode: Int, resultCode: Int, data: Intent?){
            UTSAndroid.offAppActivityResult();
            if (requestCode == 110 && resultCode == Activity.RESULT_OK && data != null) {
                val fileUri = data.getData();
                if (fileUri != null) {
                    var path = getRealPathFromURI(context, fileUri);
                    val file = File(path);
                    if (file.exists()) {
                        var upLoadFilePath = file.toString();
                        var upLoadFileName = file.getName();
                        val res = object : UTSJSONObject() {
                            var code = "0"
                            var filePath = upLoadFilePath
                            var fileName = upLoadFileName
                            var errMsg = "fileselect:ok"
                            var detail = "文件读取成功"
                        };
                        options.success?.invoke(res);
                        options.complete?.invoke(res);
                    } else options.fail?.invoke(object : UTSJSONObject() {
                        var code = "1002"
                        var errMsg = "fileselect:fail"
                        var detail = "文件不存在"
                    });
                }
            }
        });
    } else options.fail?.invoke(object : UTSJSONObject() {
        var code = "1003"
        var errMsg = "fileselect:fail"
        var detail = "AppContext不存在"
    });
}
fun getRealPathFromURI(context: Context, uri: Uri): String {
    val isKitKat = Build.VERSION.SDK_INT >= Build.VERSION_CODES.KITKAT;
    if (isKitKat) {
        if (DocumentsContract.isDocumentUri(context, uri)) {
            if (isExternalStorageDocument(uri)) {
                val docId: String = DocumentsContract.getDocumentId(uri);
                val split: UTSArray<String> = docId.split(":");
                console.log(docId, split, " at uni_modules/lemon-filePicker/utssdk/app-android/index.uts:131");
                if ("primary".equals(split[0])) return Environment.getExternalStorageDirectory().getAbsolutePath() + "/" + split[1];
                else return "/storage/" + split[0] + "/" + split[1];
            } else if (isDownloadsDocument(uri)) {
                console.log("文件选择器：【下载目录】DownloadsProvider", " at uni_modules/lemon-filePicker/utssdk/app-android/index.uts:142");
                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.Q) return saveFileFromUri(context, uri);
                val downloadPath: String = Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_DOWNLOADS).getAbsolutePath();
                val fileName: String = getFileName(context, uri);
                return downloadPath + "/" + fileName;
            } else if (isMediaDocument(uri)) {
                var docId: String = DocumentsContract.getDocumentId(uri);
                var split: UTSArray<String> = docId.split(":");
                var type: String = split[0];
                var contentUri: Uri? = null;
                if ("image".equals(type)) contentUri = MediaStore.Images.Media.EXTERNAL_CONTENT_URI;
                else if ("video".equals(type)) contentUri = MediaStore.Video.Media.EXTERNAL_CONTENT_URI;
                else if ("audio".equals(type)) contentUri = MediaStore.Audio.Media.EXTERNAL_CONTENT_URI;
                else if ("document".equals(type)) {
                    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.Q) return saveFileFromUri(context, uri);
                    return "";
                }
                val selection: String = "_id=?";
                val selectionArgs: UTSArray<String> = utsArrayOf(
                    split[1]
                );
                if (contentUri != null) return getDataColumn(context, contentUri, selection, selectionArgs);
            }
        } else if ("content".equals(uri.getScheme())) return getDataColumn(context, uri, null, null);
        else if ("file".equals(uri.getScheme())) return uri.getPath() as String;
    }
    return "";
}
fun isExternalStorageDocument(uri: Uri): Boolean {
    return "com.android.externalstorage.documents".equals(uri.getAuthority());
}
fun isDownloadsDocument(uri: Uri): Boolean {
    return "com.android.providers.downloads.documents".equals(uri.getAuthority());
}
fun isMediaDocument(uri: Uri): Boolean {
    return "com.android.providers.media.documents".equals(uri.getAuthority());
}
fun getDataColumn(context: Context, uri: Uri, selection: String?, selectionArgs: UTSArray<String>?): String {
    var column = "_data";
    var projection = arrayOf<String>(column);
    var cursor: Cursor? = null;
    val _selectionArgs = if (selectionArgs != null) {
        selectionArgs.toTypedArray();
    } else {
        null;
    }
    ;
    try {
        cursor = context.getContentResolver().query(uri, projection, selection, _selectionArgs, null);
        if (cursor != null && cursor.moveToFirst()) {
            val column_index = cursor.getColumnIndexOrThrow(column);
            return cursor.getString(column_index);
        }
    }
     catch (e: Throwable) {
        console.log(e, " at uni_modules/lemon-filePicker/utssdk/app-android/index.uts:227");
    }
     finally{
        if (cursor != null) cursor.close();
    }
    return "";
}
fun getFileName(context: Context, uri: Uri): String {
    var projection = arrayOf(MediaStore.Images.ImageColumns.DISPLAY_NAME);
    var cursor = context.getContentResolver().query(uri, projection, null, null, null);
    try {
        if (cursor != null && cursor.moveToFirst()) {
            var name_col_index = cursor.getColumnIndex(projection[0]);
            return cursor.getString(name_col_index);
        }
    }
     catch (e: Throwable) {
        console.log(e, " at uni_modules/lemon-filePicker/utssdk/app-android/index.uts:246");
    }
     finally{
        cursor?.close();
    }
    return "";
}
fun saveFileFromUri(context: Context, uri: Uri): String {
    var file: File;
    val contentResolver: ContentResolver = context.getContentResolver();
    val cursor: Cursor? = contentResolver.query(uri, null, null, null, null);
    if (cursor != null && cursor.moveToFirst()) {
        val displayName = getFileName(context, uri);
        try {
            val kIs = contentResolver.openInputStream(uri);
            if (kIs != null) {
                val file1: File = File(context.getExternalCacheDir()?.getAbsolutePath() + "/" + System.currentTimeMillis());
                if (!file1.exists()) file1.mkdir();
                val cache: File = File(file1.getPath(), displayName);
                val fos = FileOutputStream(cache);
                FileUtils.copy(kIs, fos);
                file = cache;
                fos.close();
                kIs.close();
                return file.getAbsolutePath();
            }
        }
         catch (e: Throwable) {
            console.log(e, " at uni_modules/lemon-filePicker/utssdk/app-android/index.uts:277");
        }
    }
    return "";
}
open class InfoOptionsJSONObject : UTSJSONObject() {
    open var permission: Boolean? = null;
    open var mimetype: String? = null;
    open var success: UTSCallback? = null;
    open var fail: UTSCallback? = null;
    open var complete: UTSCallback? = null;
}
fun fileSelectByJs(options: InfoOptionsJSONObject) {
    return fileSelect(InfoOptions(permission = options.permission, mimetype = options.mimetype, success = fun(res: Any): Unit {
        options.success?.invoke(res);
    }
    , fail = fun(res: Any): Unit {
        options.fail?.invoke(res);
    }
    , complete = fun(res: Any): Unit {
        options.complete?.invoke(res);
    }
    ));
}
